package jfd.ale;

class FlowResource extends ConstraintsContainer
{
   private FlowDirection flowDirection = null;
   private Node start                  = null;
   private Node end                    = null;
   private Node coord                  = null;

   public FlowResource(
      FlowDirection flowDirection,
      Node start,
      Node end,
      Node coord
   )
   {
      this.flowDirection = flowDirection;
      this.start         = start;
      this.end           = end;
      this.coord         = coord;
   }

   public Constraints[] getConstraints()
   {
      Constraints array[] = super.getConstraints();

      if(array.length > 0)
      {
         // The size of the gap between each component is:
         //    HORIZONTAL:
         //    ((end - start) - (comp1.w + comp2.w + ...)) / (nComponents + 1)
         // 
         //    VERTICAL:
         //    ((end - start) - (comp1.h + comp2.h + ...)) / (nComponents + 1)

         StringBuffer sb = 
            new StringBuffer("((" + end).append('-').append(start + ")");

         sb.append('-').append('(');

         boolean isHorizontal = (flowDirection == FlowDirection.HORIZONTAL);

         for(int i = 0; i < array.length; i ++)
         {
            String name = array[i].getName();

            if(i > 0)
               sb.append('+');

            sb.append(name).append('.').append(isHorizontal ? 'w' : 'h');
         }

         sb.append("))").append('/').append(array.length + 1);

         // The center of each component is:
         //    if first component
         //       start + gap + this.w / 2
         //    else
         //       previous_component.r + gap + this.w / 2
         // or
         //    if first component
         //       start + gap + this.h / 2
         //    else
         //       previous_component.b + gap + this.h / 2

         sb.append('+').append("this.").append(isHorizontal ? 'w' : 'h');
         sb.append("/2");

         String sbString = sb.toString();

         String location       = null;
         String locationString = null;

         for(int i = 0; i < array.length; i ++)
         {
            if(i == 0)
               location = start.toString();
            else
            {
               location = array[i - 1].getName() + "." +
                  (isHorizontal ? "r" : "b");
            }

            if(isHorizontal)
               locationString = location + "+" + sbString + "," + coord;
            else
               locationString = coord + "," + location + "+" + sbString;

            try
            {
               NodePair pair = Parser.parseNodePair(locationString);

               array[i] =
                  new Constraints(
                     array[i].getName(),
                     array[i].getWidth(),
                     array[i].getHeight(),
                     pair.getFirst(),
                     pair.getSecond(),
                     array[i].getAlign()
                  );
            }
            catch(Exception e)
            {
               // If we are here, it is an internal error -- the
               // String we created for "locationString" had a
               // syntax error

               System.out.println(
                  "FlowResource - problem parsing: [" + locationString + "]"
               );

               System.out.println(e.getMessage());
            }
         }
      }

      return(array);
   }
}
